<?php
defined("SAWANNA") or die();

switch (DB_TYPE)
{
	case 'MySQL':
		require_once(ROOT_DIR.'/sawanna/mysql.inc.php');
		break;
	case 'SQLite':
		require_once(ROOT_DIR.'/sawanna/sqlite.inc.php');
		break;
}

require_once(ROOT_DIR.'/sawanna/global.php');
require_once(ROOT_DIR.'/sawanna/session.inc.php');
require_once(ROOT_DIR.'/sawanna/language.inc.php');
require_once(ROOT_DIR.'/sawanna/tpl.inc.php');
require_once(ROOT_DIR.'/sawanna/user.inc.php');
require_once(ROOT_DIR.'/sawanna/node.inc.php');
require_once(ROOT_DIR.'/sawanna/navigator.inc.php');
require_once(ROOT_DIR.'/sawanna/mail.inc.php');
require_once(ROOT_DIR.'/sawanna/utf8.inc.php');
require_once(ROOT_DIR.'/sawanna/log.inc.php');
require_once(ROOT_DIR.'/sawanna/captcha.inc.php');

class CSawanna
{
	var $query;
	var $db;
	var $session;
	var $language;
	var $tpl;
	var $user;
	var $node;
	var $navigator;
	var $mail;
	var $utf8;
	var $log;
	var $captcha;
	var $environment;
	var $title;
	var $message;
	var $error;
	var $ok;
	
	function CSawanna()
	{
		define("VERSION",'2.8.5');
		$this->query=array();
		$this->db=null;
		$this->session=null;
		$this->language=null;
		$this->tpl=null;
		$this->user=null;
		$this->node=null;
		$this->navigator=null;
		$this->mail=new CMail();
		$this->utf8=new CUTF8();
		$this->log=null;
		$this->captcha=null;
		$this->environment=array();
		$this->title='Powered by Sawanna CMS';
		$this->message="";
		$this->error="";
		$this->ok=false;
	}
	
	function init()
	{
		global $sawanna_languages,$sawanna_workspaces,$sawanna_templates,$sawanna_modules,$sawanna_ajax_components,$sawanna_rss_components,$sawanna_admin_components,$sawanna_admin_panel_elements;
		
		DEBUG_MODE ? error_reporting(E_ALL) : error_reporting(0);
		
		$content_type='text/html';
		
		$this->db=new CDatabase();
		if (!$this->db->init()) { DEBUG_MODE ? die($this->db->error) : die(); }
		$this->environment['db']=&$this->db;
		
		$this->log=new CLog($this->environment['db']);
		if (ANTIBOT && $this->log->is_bot()) { DEBUG_MODE ? die('Bot detected.<br />'.$this->log->error) : die(); }
		$this->environment['log']=&$this->log;
		
		$this->validate_inputs();
		
		$slogan=array();
		$this->db->query("select * from ".DB_PREFIX."_configuration");
		while ($constant=$this->db->fetch_array())
		{
			if ($constant['name'] == 'SITE_SLOGAN')
			{
				$slogan[$constant['mark']]=$constant['value'];
				continue;
			}
			if (!defined($constant['name'])) define($constant['name'],$constant['value']);
		}
		
		$this->import_globals();
		
		$this->language=new CLanguage($this->environment['db']);
		
		if (count($this->language->languages) > 1)
		{
			$this->query=explode("/",urldecode($_SERVER['QUERY_STRING']),3);
		} else 
		{
			$this->query=explode("/",urldecode($_SERVER['QUERY_STRING']),2);
			array_unshift($this->query,'');
		}
		
		if (!isset($this->query[1]) || empty($this->query[1])) $this->query[1]=FRONT_PAGE_WORKSPACE;
		if ($this->query[1] != 'ajax' && $this->query[1] != 'captcha' && $this->query[1] != 'rss' && !array_key_exists($this->query[1],$sawanna_workspaces)) { DEBUG_MODE ? $this->message('Unknown workspace : '."'".$this->query[1]."'") : die(); }
		$this->environment['workspace']=&$this->query[1];
		
		if ($this->query[1] == 'captcha') $content_type='image/jpeg';
		if ($this->query[1] == 'rss') $content_type='xml';
		
		$this->session=new CSession($this->environment['db'],$this->environment['workspace']);
		if (!$this->session->init()) { DEBUG_MODE ? $this->message($this->session->error) : die(); }
		$this->environment['session']=&$this->session;
		
		if (!isset($this->query[0])) $this->query[0]='';
		if (!$this->language->init($this->query[0])) { DEBUG_MODE ? $this->message($this->language->error) : die(); }
		$this->environment['language']=&$this->language;
		
		if (isset($slogan[$this->language->current]))  $slogan=$slogan[$this->language->current]; elseif (isset($slogan[DEFAULT_LANGUAGE]))  $slogan=$slogan[DEFAULT_LANGUAGE]; else $slogan='';
		define('SITE_SLOGAN',$slogan);
		unset($slogan);
		
		if (isset($this->query[2]) && !empty($this->query[2]))
		{
			if (($amp_pos=strpos($this->query[2],'&'))!==false) $this->query[2]=substr($this->query[2],0,$amp_pos);
			if (!empty($this->query[2]) && ($last_slash_pos=strpos($this->query[2],'/',strlen($this->query[2])-1))!==false) $this->query[2]=substr($this->query[2],0,$last_slash_pos);
		} else {
			$this->query[2]='';
		}
		$this->environment['query']=&$this->query[2];
		
		$this->tpl=new CTpl($this->query[1],$this->query[2],$this->environment['db'],$this->environment['language']);
		if ($this->query[1] != 'ajax' && $this->query[1] != 'captcha' && $this->query[1] != 'rss' && !$this->tpl->load()) { DEBUG_MODE ? $this->message($this->tpl->error) : die(); }
		$this->environment['tpl']=&$this->tpl;
		
		$this->environment['utf8']=&$this->utf8;
		$this->user=new CUser($this->environment['db'],$this->environment['session'],$this->environment['utf8']);
		$this->user->identify();
		$this->environment['user']=&$this->user;
		
		$this->node=new CNode($this->environment['db'],$this->environment['language'],$this->environment['user']);
		$this->environment['node']=&$this->node;
		
		$this->navigator=new CNavigator($this->environment['workspace'],$this->environment['query'],$this->environment['db'],$this->environment['language'],$this->environment['user']);
		$this->environment['navigator']=&$this->navigator;
		
		$this->captcha=new CCaptcha($this->environment['session']);
		$this->environment['captcha']=&$this->captcha;
		
		$this->environment['mail']=&$this->mail;
		$this->environment['ok']=&$this->ok;
		
		switch ($content_type)
		{
			case 'image/jpeg' :
				header("Content-Type: image/jpeg");
				break;
			case 'xml' :
				header('Content-type: application/xml');
				break;
			default :
				header("Content-Type: text/html; charset=utf-8");
				break;
		}
	}
	
	function validate_inputs()
	{
		global $bad_words;
		
		foreach ($bad_words as $key=>$word)
		{
			if (strlen($word)>1 && $word!='\\x00') $bad_words[$key]='\b'.$word.'\b';
		}
		$pattern='/'.implode('|',$bad_words).'/i';
		
		$gpc=$_SERVER['QUERY_STRING'].
			 ' '.implode(' ',$_GET);
			 
		if (!IGNORE_POST_BAD_WORDS)
		$gpc.=' '.implode(' ',$_POST);
		
		$gpc.=' '.implode(' ',$_COOKIE);
		if (isset($_SERVER['HTTP_USER_AGENT'])) 
		$gpc.=' '.$_SERVER['HTTP_USER_AGENT'];
		
		if (isset($_SERVER['HTTP_ACCEPT'])) 
		$gpc.= ' '.$_SERVER['HTTP_ACCEPT'];
		
		if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) 
		$gpc.=' '.$_SERVER['HTTP_ACCEPT_LANGUAGE'];
		
		if (isset($_SERVER['HTTP_ACCEPT_ENCODING'])) 
		$gpc.=' '.$_SERVER['HTTP_ACCEPT_ENCODING'];
			 
		if (isset($_SERVER['HTTP_ACCEPT_CHARSET']))
		$gpc.=' '.$_SERVER['HTTP_ACCEPT_CHARSET'];
		
		$gpc.=' '.$_SERVER['REMOTE_ADDR'].
			  ' '.$_SERVER['REMOTE_PORT'].
			  ' '.$_SERVER['REQUEST_URI'];
			 
		if (isset($_SERVER['HTTP_REFERER'])) 
		$gpc.=' '.$_SERVER['HTTP_REFERER'];
		
		$x_forwarded_for=getenv('HTTP_X_FORWARDED_FOR');
		if (!empty($x_forwarded_for)) 
		$gpc.=' '.$x_forwarded_for;
		
		$matches=array();
		preg_match($pattern,$gpc,$matches);
		if (!empty($matches))
		{
			 DEBUG_MODE ? $this->message('Bad word or simbol detected : '.$matches[0]) : die();
		}
	}
	
	function construct()
	{
		global $sawanna_workspaces,$sawanna_admin_panel_elements;
		
		if ($this->user->group != 1 && $this->query[1] == ADMIN_WORKSPACE)
		{
			$content=$this->build_auth_page();
			echo $this->language->parse_words($content);
			$this->session->fix();
			$this->ok=true;
			return ;
		}
		
		!empty($this->query[1]) ? $workspace=$this->query[1] : $workspace=FRONT_PAGE_WORKSPACE;
		if (array_key_exists($workspace,$sawanna_workspaces))
		{
			if (is_array($sawanna_workspaces[$workspace]))
			{
				$contents=array();
				foreach ($this->tpl->components as $component)
				{
					if (array_key_exists($component,$sawanna_workspaces[$workspace]))
					{
						if (is_array($sawanna_workspaces[$workspace][$component]))
						{
							$contents[$component]='';
							foreach ($sawanna_workspaces[$workspace][$component] as $module)
							{
								$contents[$component].=$this->call_module($module);
							}
						} else 
						{
							$contents[$component]=$this->call_module($sawanna_workspaces[$workspace][$component]);
						}
					}
				}
				$this->tpl->render($contents,$this->title,$this->message,$this->error);
				if ($this->user->group==1)
				{
					$sawanna_admin_panel_elements['sawanna']=$this->build_url(ADMIN_WORKSPACE);
					
					$admin_elems=array_keys($sawanna_admin_panel_elements);
					foreach ($admin_elems as $key)
					{
						$sawanna_admin_panel_elements[$key]['url']=$this->build_url($sawanna_admin_panel_elements[$key]['url']);
					}
					
					$sawanna_admin_panel_elements['logout']=$this->build_url('user/logout');
					
					$this->tpl->construct_admin_panel();
				}
				
				$this->tpl->paste_meta($this->navigator,$this->node);
				echo $this->language->parse_words($this->tpl->html);
				
				$this->session->fix();
				$this->ok=true;
			} else { DEBUG_MODE ? die('Workspace map must be an array') : die(); }
		} elseif ($this->query[1] == 'ajax')
		{  
			if (!empty($this->query[2]))
			{
				$content=$this->call_module($this->query[2],true);
				echo $this->language->parse_words($content);
				
				$this->session->fix();
				$this->ok=true;
			}
		} elseif ($this->query[1] == 'rss')
		{  
			if (!empty($this->query[2]))
			{
				$content=$this->call_module($this->query[2],false,true);
				echo $this->language->parse_words($content);
				
				$this->session->fix();
				$this->ok=true;
			}
		} elseif ($this->query[1] == 'captcha')
		{
			header("Cache-Control: no-cache, must-revalidate");
			header("Expires: ".date('r',time()-3600)); 
	
			if (isset($_SERVER['HTTP_REFERER']))
			{
				$this->captcha->generate();
				$this->session->fix();
				$this->ok=true;
			}
		} else { DEBUG_MODE ? die('Unknown workspace : '."'".$this->query[1]."'") : die(); }
	}
	
	function call_module($module,$ajax=false,$rss=false)
	{
		global $sawanna_modules,$sawanna_ajax_components,$sawanna_admin_components,$sawanna_rss_components;
		
		if (in_array($module,$sawanna_admin_components) && $this->user->group != 1) return;
		
		$output='';
		if ((!$ajax && array_key_exists($module,$sawanna_modules)) || ($ajax && array_key_exists($module,$sawanna_ajax_components)) || ($rss && array_key_exists($module,$sawanna_rss_components)))
		{
			if ((!$ajax && file_exists(ROOT_DIR.'/module/'.$sawanna_modules[$module])) || ($ajax && file_exists(ROOT_DIR.'/module/'.$sawanna_ajax_components[$module])) || ($rss && file_exists(ROOT_DIR.'/module/'.$sawanna_rss_components[$module])))
			{
				ob_start();
				!$ajax ? include(ROOT_DIR.'/module/'.$sawanna_modules[$module]) : include(ROOT_DIR.'/module/'.$sawanna_ajax_components[$module]);
				$init=$module.'_init';
				$exec=$module.'_exec';
				$clear=$module.'_clear';
				if (function_exists($init)) $init($this->environment);
				if (function_exists($exec)){
					list($message,$error,$title)=$exec();
					if (!empty($message)) $this->message=$message;
					if (!empty($error)) $this->error=$error;
					if (!empty($title)) $this->title=$title;
					unset($message,$error,$title);
				}
				if (function_exists($clear)) $clear();
				$output=ob_get_contents();
				ob_end_clean();
			} elseif ((!$ajax && file_exists(ROOT_DIR.'/addons/'.$sawanna_modules[$module])) || ($ajax && file_exists(ROOT_DIR.'/addons/'.$sawanna_ajax_components[$module])) || ($rss && file_exists(ROOT_DIR.'/addons/'.$sawanna_rss_components[$module])))
			{
				ob_start();
				!$ajax ? include(ROOT_DIR.'/addons/'.$sawanna_modules[$module]) : include(ROOT_DIR.'/addons/'.$sawanna_ajax_components[$module]);
				$init=$module.'_init';
				$exec=$module.'_exec';
				$clear=$module.'_clear';
				if (function_exists($init)) $init($this->environment);
				if (function_exists($exec)){
					list($message,$error,$title)=$exec();
					if (!empty($message)) $this->message=$message;
					if (!empty($error)) $this->error=$error;
					if (!empty($title)) $this->title=$title;
					unset($message,$error,$title);
				}
				if (function_exists($clear)) $clear();
				$output=ob_get_contents();
				ob_end_clean();
			} else { DEBUG_MODE ? $this->message("Module '".$module."' not found.") : die(); }
		} else { DEBUG_MODE ? $this->message("Module '".$module."' not registered.") : die(); }
		return $output;
	}
	
	function fix_url($url,$force_sid=false,$strip_sid=false)
	{
		global $searchBots;
		
		if (!$force_sid && CLEAN_URL && isset($this->session->var['clean_url']))
		{
			if ($this->session->var['clean_url']=='true')
			{
				if (strpos($url,HOME.'/index.php?')!==false)
				{
					$url=HOME.'/'.str_replace(HOME.'/index.php?','',$url);
				}
			} elseif ($this->session->var['clean_url']=='false')
			{
				if (strpos($url,HOME.'/index.php')===false)
				{
					$url=HOME.'/index.php?'.str_replace(HOME.'/','',$url);
				}
			}
		}
		
		if (($force_sid || !isset($this->session->var['cookie_enabled']) || $this->session->var['cookie_enabled']=='false') && !$strip_sid)
		{
			if (strpos($url,"?")===false)
			{
				if (strpos($url,"/",strlen($url)-1)===false)
				{
					$url.='/?/';
				} else 
				{
					$url.='?/';
				}
			} else 
			{
				if (strpos($url,'&',strpos($url,'?'))===false)
				{
					if (strpos($url,"/",strlen($url)-1)===false)
					{
						$url.='/';
					}
				}
				
				if (strpos($url,'&first=true')!==false)
				{
					$url=str_replace('&first=true','',$url);
				}
				
				if (SESSION_EMULATE)
				{
					if (($sid_pos=strpos($url,'&sid='))!==false)
					{
						$url=substr($url,0,$sid_pos);
					}
				} else 
				{
					if (($sid_pos=strpos($url,'&PHPSESSID='))!==false)
					{
						$url=substr($url,0,$sid_pos);
					}
				}
			}
			
			if(!SESSION_EMULATE)
			{
				if (!isset($_SERVER['HTTP_USER_AGENT']) || !in_array($_SERVER['HTTP_USER_AGENT'],$searchBots))
					$url.='&PHPSESSID='.session_id();
			} else
			{
				if (!isset($_SERVER['HTTP_USER_AGENT']) || !in_array($_SERVER['HTTP_USER_AGENT'],$searchBots))
					$url.='&sid='.$this->session->sid;
			}
		} else 
		{
			if (SESSION_EMULATE)
			{
				if (($sid_pos=strpos($url,'&sid='))!==false)
				{
					$url=substr($url,0,$sid_pos);
				}
			} else 
			{
				if (($sid_pos=strpos($url,'&PHPSESSID='))!==false)
				{
					$url=substr($url,0,$sid_pos);
				}
			}
		}
		
		return $url;
	}
	
	function build_url($url,$force_sid=false,$strip_sid=false)
	{
		if (isset($_GET['gzip']) && $_GET['gzip']=='false') $force_sid=true;
		
		if (($force_sid || !isset($this->session->var['clean_url']) || $this->session->var['clean_url']=='false') && !$strip_sid)
		{
			if (count($this->language->languages) > 1)
				$url=HOME.'/index.php?'.$this->language->current.'/'.$url;
			else 
				$url=HOME.'/index.php?'.$url;
				
			$url=$this->fix_url($url,$force_sid);
		} elseif ($strip_sid)
		{
			if (count($this->language->languages) > 1)
				$url=HOME.'/index.php?'.$this->language->current.'/'.$url;
			else 
				$url=HOME.'/index.php?'.$url;
				
			$url=$this->fix_url($url,false,true);
		} else 
		{
			if (count($this->language->languages) > 1)
				$url=HOME.'/'.$this->language->current.'/'.$url;
			else 
				$url=HOME.'/'.$url;
		}
		
		if (isset($_GET['gzip']) && $_GET['gzip']=='false')
		{
			$url.='&gzip=false';
		}
		
		return $url;
	}
	
	function build_anchor($link,$text,$class='')
	{
		if (!empty($class)) $class=' class="'.$class.'"';
		
		!empty($link) ? $link=' href="'.$this->build_url($link).'"' : '';
		
		return '<a'.$class.$link.'>'.$text.'</a>';
	}
	
	function build_form($url,$elements,$width='100%',$descriptions='',$attributes='',$custom_elements='',$class='',$shadow=false,$id='',$target='')
	{
		$url=$this->build_url($url);
		
		if (!empty($id)) $id=' id="'.$id.'"';
		if (!empty($target)) $target=' target="'.$target.'"';
		
		if (in_array('file',$elements))
			$html='<form'.$id.$target.' action="'.$url.'" method="post" enctype="multipart/form-data">';
		else 
			$html='<form'.$id.$target.' action="'.$url.'" method="post">';
			
		$class=(!empty($class)) ? ' class="'.$class.'"' : '';
		$html.='<table'.$class.' width="'.$width.'" cellpadding="2" cellspacing="2"><tr><td width="50%" height="10"></td><td width="50%"></td></tr>';
		
		foreach ($elements as $key=>$element)
		{
			if (!empty($descriptions) && isset($descriptions[$key]) && !empty($descriptions[$key]))
			{
				$description=$descriptions[$key];
				$html.='<tr><td align="right" class="txt">'.$description.'</td><td align="left">';
			} else 
			{
				$html.='<tr><td colspan="2" align="center">';
			}
				
			
			if (!empty($attributes) && isset($attributes[$key]) && !empty($attributes[$key]))
				$attribute=$attributes[$key];
			else 
				$attribute='';
				
			switch ($element)
			{
				case 'text' :
					$html.='<input type="text" '.$attribute.' />';
					break;
				case 'password' :
					$html.='<input type="password" '.$attribute.' />';
					break;
				case 'hidden' :
					$html.='<input type="hidden" '.$attribute.' />';
					break;
				case 'file' :
					$html.='<input type="file" '.$attribute.' />';
					break;
				case 'textarea' :
					$html.='<textarea '.$attribute.'></textarea>';
					break;
				case 'checkbox' :
					$html.='<input type="checkbox" '.$attribute.' />';
					break;
				case 'radio' :
					$html.='<input type="radio" '.$attribute.' />';
					break;
				case 'custom' :
					if (!empty($custom_elements) && isset($custom_elements[$key]) && !empty($custom_elements[$key]))
						$html.=$custom_elements[$key];
					break;
				case 'submit' :
					$html.='<input type="submit" '.$attribute.' />';
					break;
			}
			
			$html.='</td></tr>';	
		}
		
		$html.='<tr><td height="10"></td><td></td></tr></table>';
		if ($shadow) $html.='<div style="width:'.$width.'" class="shadow_1"></div>';
		$html.='</form>';
		
		return $html;
	}
	
	function build_wysiwyg($id,$type='common',$component='')
	{
		$this->db->query("select value from ".DB_PREFIX."_global where mark='editor' and name='sawanna_wysiwyg'");
		$glob=$this->db->fetch_array();
		
		if (empty($glob)) return ;
		
		$wysiwyg=unserialize($glob['value']);
		
		$html='';
		switch ($type)
		{
			case 'common' :
				$html.=$wysiwyg['common'];
				break;
			case 'alter' :
				if (!empty($component))
				{
					$html.=$wysiwyg['alter'];
					$html=str_replace('<component>',$component,$html);
				}
				break;
		}

		$html=str_replace('<id>',$id,$html);
		$html=str_replace('<theme>',THEME,$html);
		
		unset($glob);
		$this->db->query("select value from ".DB_PREFIX."_global where mark='editor' and name='sawanna_wysiwyg_langs'");
		$glob=$this->db->fetch_array();
		
		if (!empty($glob))
		{
			$wysiwyg_langs=unserialize($glob['value']);
			
			if (in_array($this->language->current,$wysiwyg_langs))
			{
				$html=str_replace('<lng>',$this->language->current,$html);
			} elseif (in_array(DEFAULT_LANGUAGE,$wysiwyg_langs))
			{
				$html=str_replace('<lng>',DEFAULT_LANGUAGE,$html);
			} else 
			{
				$html=str_replace('<lng>',$wysiwyg_langs[0],$html);
			}
		}
		
		return $html;
	}
	
	function build_limiter($count,$limit,$link,$query=1,$get=false)
	{
		if ($query<1) return ;
		
		$html='';
		$c=ceil($count/$limit);

		if ($query>$c) return ;
			
		$begin=($query-9)>0 ? $query-9 : 0;
		
		$end=($query+9)<$c ? $query+9 : $c;
		
		if ($begin!=0)
		{
			if (!$get)
				$html.= $this->build_anchor($link.'/1','...').'&nbsp;&nbsp;';
			else 
				$html.= $this->build_anchor($link.'&get=1','...').'&nbsp;&nbsp;';
		}
		if ($query>1)
		{
			if (!$get)
				$html.= $this->build_anchor($link.'/'.($query-1),'&laquo;').'&nbsp;&nbsp;';
			else 
				$html.= $this->build_anchor($link.'&get='.($query-1),'&laquo;').'&nbsp;&nbsp;';
		}
		
		for ($i=$begin;$i<$end;$i++)
		{
			if ($i+1==$query)
				$html.= '<span class="limiter-current"><b>'.($i+1).'</b></span>&nbsp;';
			else
			{
				if (!$get)
					$html.= $this->build_anchor($link.'/'.($i+1),'<b>'.($i+1).'</b>','limiter').'&nbsp;';
				else 
					$html.= $this->build_anchor($link.'&get='.($i+1),'<b>'.($i+1).'</b>','limiter').'&nbsp;';
			}
		}
		
		if ($query<$c) 
		{
			if (!$get)
				$html.= '&nbsp;&nbsp;'.$this->build_anchor($link.'/'.($query+1),'&raquo;');
			else 
				$html.= '&nbsp;&nbsp;'.$this->build_anchor($link.'&get='.($query+1),'&raquo;');
		}
		if ($end<$c) 
		{
			if (!$get)
				$html.= '&nbsp;&nbsp;'.$this->build_anchor($link.'/'.$c,'...');
			else 
				$html.= '&nbsp;&nbsp;'.$this->build_anchor($link.'&get='.$c,'...');
		}
		
		return $html;
	}
	
	function build_auth_page()
	{
		$elems=array(
					'text',
					'password',
					'submit'					
					);
		$attr=array(
					'name="login" style="width:100%"',
					'name="password" style="width:100%"',
					'value="[string]send[/string]" style="width:100px"'
					);
		$descr=array(
					'[string]login[/string]',
					'[string]password[/string]',
					' '
					);
			
		$html='<html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" /><title>[string]authorization[/string]</title><link rel="stylesheet" type="text/css" href="'.HOME.'/theme/'.THEME.'/style.css" /></head><body>';
		$html.='<table width="100%" height="100%"><tr><td width="100%" height="100%" align="center">';
		$html.=$this->build_form('user',$elems,'300px',$descr,$attr,'','form');
		$html.='</td></tr></table>';
		$html.='</body></html>';
		
		return $html;
	}
	
	function redirect($dest)
	{
		if (isset($this->session->var['cookie_enabled']) && $this->session->var['cookie_enabled']=='false' && ($dest==HOME || $dest==HOME.'/' || $dest==HOME.'/index.php'))
		{
			$dest=$this->build_url(FRONT_PAGE_WORKSPACE);
		}
		
		$dest=$this->fix_url($dest);
		
		$this->session->fix();
		$this->ok=true;
		
		if ($this->ok) header("Location: ".$dest);
		//if ($this->ok) echo $this->tpl->generate_redirect_page($dest);
		exit();
	}
	
	function import_globals()
	{
		global $sawanna_languages,$sawanna_workspaces,$sawanna_templates,$sawanna_modules,$sawanna_ajax_components,$sawanna_rss_components,$sawanna_admin_components,$sawanna_admin_panel_elements;
		
		$this->db->query("select * from ".DB_PREFIX."_global order by id");
		while ($glob=$this->db->fetch_array()) {
			if (!isset($$glob['name']) || !is_array($$glob['name'])) continue;

			$glob_val=unserialize($glob['value']);
			if (!is_array($glob_val)) continue;

			foreach ($glob_val as $key=>$value)
			{ 
				$glob_ref=&$$glob['name'];
				if (!isset($glob_ref[$key])) continue;
				
				if (is_array($glob_ref[$key]) && is_array($value))
				{ 
					foreach ($value as $_key=>$_value)
					{
						if (is_array($glob_ref[$key][$_key]))
						{
							if (is_array($_value))
							{
								$diff=array_diff($_value,$glob_ref[$key][$_key]);
								$glob_ref[$key][$_key]=array_merge($glob_ref[$key][$_key],$diff);
							} elseif (!in_array($_value,$glob_ref[$key][$_key])) 
							{
								array_push($glob_ref[$key][$_key],$_value);
							}
						} else 
						{
							if (is_array($_value))
							{
								$arr_dist=array($glob_ref[$key][$_key]);
								$diff=array_diff($_value,$arr_dist);
								$glob_ref[$key][$_key]=array_merge($arr_dist,$diff);
							} elseif (!in_array($_value,$glob_ref[$key][$_key])) 
							{
								$glob_ref[$key][$_key]=array($glob_ref[$key][$_key],$_value);
							}
						}
					}
				}
				
				unset($glob_val[$key]);
			}
			$$glob['name']=array_merge($$glob['name'],$glob_val);
		}
	}
	
	function message($mess)
	{
		$this->error=$mess;
		return;
	}
	
	function clear()
	{
		if (isset($this->db) && $this->db->connection_id) $this->db->close();
		unset(
			$this->query,
			$this->db,
			$this->session,
			$this->language,
			$this->tpl,
			$this->user,
			$this->node,
			$this->navigator,
			$this->mail,
			$this->log,
			$this->utf8,
			$this->captcha,
			$this->environment,
			$this->title,
			$this->message,
			$this->error,
			$this->ok
			);
	}
}
?>