<?php
####################################################################################
#NOTE! Template head and body tags should be written without spaces and in lowercase
####################################################################################

defined("SAWANNA") or die();

class CTpl
{
	var $workspace;
	var $query;
	var $db;
	var $language;
	var $template;
	var $html;
	var $components;
	var $error;
	var $javascript;
	
	function CTpl($workspace,$query,&$db,&$language)
	{
		global $sawanna_templates;
		
		$this->workspace=$workspace;
		$this->query=$query;
		$this->db=&$db;
		$this->language=&$language;
		
		if (array_key_exists($this->workspace,$sawanna_templates)) $this->template=$sawanna_templates[$this->workspace];
		$this->html="";
		$this->components=array();
		$this->error='';
		$this->javascript=array();
	}
	
	function load($name='')
	{
		empty($name) ? $name=$this->template : $name=quotemeta($name);
		if (empty($name))
		{
			$this->error="Template is not assigned.";
			return false;
		}
		if (file_exists(ROOT_DIR.'/theme/'.THEME.'/'.$name.'.tpl'))
		{
			$this->html.=file_get_contents(ROOT_DIR.'/theme/'.THEME.'/'.$name.'.tpl');
			
			$this->html=str_replace('[HOST]',HOME,$this->html);
			
			if (strpos($this->html,'js/window.js') === false) $this->javascript['window']=false; else $this->javascript['window']=true;
			if (strpos($this->html,'js/cajax.js') === false) $this->javascript['cajax']=false; else $this->javascript['cajax']=true;
			if (strpos($this->html,'js/jquery-1.3.2.min.js') === false) $this->javascript['jquery']=false; else $this->javascript['jquery']=true;

			$this->components=$this->get_components();
			
			if (DEBUG_MODE && ($body=strpos($this->html,'<body>')+6) !==false)
			{
				$this->html=substr($this->html,0,$body)."\n"."<div style=\"width:100%;height:25px;line-height:25px;background-color:FF3355;text-align:center;font-family:Arial;font-size:10pt;color:#FFFFFF\">DEBUG MODE</div>"."\n".substr($this->html,$body);
			}
			
			$this->html.="\n<!--Powered by Sawanna CMS v.".VERSION."-->";
			return true;
		} else
		{
			$this->error="Template '".$name."' not found.";
			return false;
		}
	}
	
	function get_components($name="")
	{
		if (empty($name)) $html=$this->html;
		else if (file_exists(ROOT_DIR.'/theme/'.THEME.'/'.$name.'.tpl'))
		{
			$html=file_get_contents(ROOT_DIR.'/theme/'.THEME.'/'.$name.'.tpl');
		} else
		{
			$this->error="Template '".$name."' not found.";
			return false;
		}
		
		$components=array();
		$offset=0;
		while (true)
		{
			if (($b_pos=strpos($html,"[component",$offset))===false)
				break;
			
			if (($e_pos=strpos($html,"/component]",$offset))===false)
				break;
			
			$components[]=substr($html,$b_pos+11,$e_pos-$b_pos-12);
			$offset=$e_pos+11;
		}
		return $components;
	}
	
	function paste_meta(&$navigator,&$node)
	{
	            if (!empty($this->workspace) && !empty($this->query))
            	{
            		$result=$this->db->sub_query("select keywords,name from ".DB_PREFIX."_nodes where workspace='".$this->db->escape_string($this->workspace)."' and link='".$this->db->escape_string($this->query)."' and language='".$this->db->escape_string($this->language->current)."'");
            	} elseif ($this->workspace==FRONT_PAGE_WORKSPACE)
        		{
        		           $_workspace='public';
        			$_result=$navigator->get(array('parent'=>0));
        			
        			while ($menu=$this->db->fetch_array($_result)) {
        				if (($p=strpos($menu['link'],'/'))===false) continue;
        				if (substr($menu['link'],0,$p) != $_workspace) continue;
        				
        				$_node=array('workspace'=>$_workspace,'link'=>substr($menu['link'],$p+1),'mark'=>'');
        				$result=$node->get($_node);
        				if (!empty($result)) break;
        			}
        		}
        		
        		if (!empty($result)) {
            		$_node=$this->db->fetch_array($result);
            		if (!empty($_node) && ($head=strpos($this->html,'</head>')) !==false)
            		{
            		           $meta='';
            		           
            		           if (defined('PASTE_META_KEYWORDS') && PASTE_META_KEYWORDS)
            		              $meta.='<meta name="keywords" content="'.strip_tags($_node['keywords']).'" />'."\n";
            		           
            		           if (defined('PASTE_META_DESCRIPTION') && PASTE_META_DESCRIPTION)
            		              $meta.='<meta name="description" content="'.strip_tags($_node['name']).'" />'."\n";
            		           
            		           if (!empty($meta))
            			   $this->html=substr($this->html,0,$head).$meta.substr($this->html,$head);
            		}
            	}
	}
	
	function render(&$contents,$title,$message,$error)
	{
		if (!is_array($contents))
		{
			$this->error="Can not render template. Contents argument must be an array.";
			return false;
		}
		if (empty($this->components))
		{
			$this->error="Can not render template. No components are loaded.";
			return false;
		}
		foreach ($this->components as $component)
		{
			if (isset($contents[$component]))  $content=&$contents[$component]; elseif (DEBUG_MODE) $content='{'.$component.'}'; else $content='';
			
			$pre_block='';
			$this->db->query("select code from ".DB_PREFIX."_blocks where component='".$this->db->escape_string($component)."' and ( page = '".$this->db->escape_string($this->workspace)."' or page = '".$this->db->escape_string($this->workspace.'/'.$this->query)."' or page = '' ) and weight < 0 order by weight ");
			while ($block=$this->db->fetch_array())
			{
				$pre_block.=$block['code'].'<br />';
			}
			
			$post_block='';
			$this->db->query("select code from ".DB_PREFIX."_blocks where component='".$this->db->escape_string($component)."' and ( page = '".$this->db->escape_string($this->workspace)."' or page = '".$this->db->escape_string($this->workspace.'/'.$this->query)."' or page = '' ) and weight >= 0 order by weight ");
			while ($block=$this->db->fetch_array())
			{
				$post_block.=$block['code'].'<br />';
			}
			
			$content=ini_get("magic_quotes_gpc") ? stripslashes($content) : $content;
			$pre_block=ini_get("magic_quotes_gpc") ? stripslashes($pre_block) : $pre_block;
			$post_block=ini_get("magic_quotes_gpc") ? stripslashes($post_block) : $post_block;
			
			switch ($component)
			{
				case 'title':
				           if (defined('SHOW_SITE_NAME_IN_PAGE_TITLE') && SHOW_SITE_NAME_IN_PAGE_TITLE)
					   $this->html=str_replace("[component]".$component."[/component]",SITE_NAME.': '.$title,$this->html);
				           else 
				              $this->html=str_replace("[component]".$component."[/component]",$title,$this->html);
					break;
				case 'breadcrumbs':
					$this->html=str_replace("[component]".$component."[/component]",$content,$this->html);
					break;
				case 'message':
					$content='';
					if (!empty($message)) $content.='<div style="padding:5px 20px"><div id="message"><div style="padding:5px">'.$message.'</div></div></div>';
					if (!empty($error)) $content.='<div style="padding:5px 20px"><div id="error"><div style="padding:5px">'.$error.'</div></div></div>';
					$this->html=str_replace("[component]".$component."[/component]",$content,$this->html);
					break;
				case 'footer':
				           if (defined('PROMOTE') && PROMOTE) $post_block.='<div class="promotion">Powered by <a href="http://sawanna.org">Sawanna CMS</a></div>';
					$this->html=str_replace("[component]".$component."[/component]",$pre_block.'<div id="'.$component.'">'.$content."</div>".$post_block,$this->html);
					break;
				default:
					if ($this->workspace==ADMIN_WORKSPACE && $component=='body')
						$this->html=str_replace("[component]".$component."[/component]",'<div id="'.$component.'" style="background-color:#FFFFFF;border:1px solid #DDDDDD;background-image:url(\'/sawanna/images/palma.jpg\');background-position:bottom right;background-repeat:repeat-y;padding:10px">'.$content."</div>",$this->html);
					else
						$this->html=str_replace("[component]".$component."[/component]",$pre_block.'<div id="'.$component.'">'.$content."</div>".$post_block,$this->html);
					break;
			}
		}
		
		return true;
	}
	
	function generate_redirect_page($url)
	{
		//$code='<html><head><meta http-equiv="Refresh" content="0; url='.$url.'" /></head><body></body></html>';
		if (DEBUG_MODE)
		{
			$code='<script language="JavaScript">window.setTimeout(\'window.location.href="'.$url.'"\',1000);</script><noscript><div style="width:100%;height:30px;line-height:30px;font-size:14px;font-family:Arial;color:#000000;text-align:center;background-color:#f99199">JavaScript enabled is required for this <a href="'.$url.'">page</a> !</div></noscript>';
		} else 
		{
			$code='<script language="JavaScript">window.location.href="'.$url.'";</script><noscript><div style="width:100%;height:30px;line-height:30px;font-size:14px;font-family:Arial;color:#000000;text-align:center;background-color:#f99199">JavaScript enabled is required for this <a href="'.$url.'">page</a> !</div></noscript>';
		}
		return $code;
	}
	
	function generate_window($width,$height,&$content,$title,$display=true,$hideonly=false)
	{
		$html='';
		if (!$this->javascript['window'])
		{
		             $html.='<script language="JavaScript">';
		             $html.='SawannaWindowTopLeft = new Image();';
                                       $html.='SawannaWindowTopMiddle = new Image();';
                                       $html.='SawannaWindowBtn = new Image();';
                                       $html.='SawannaWindowBtnOver = new Image();';
                                       $html.='SawannaWindowTopLeft.src = \''.HOME.'/sawanna/images/window_left.gif\';';
                                       $html.='SawannaWindowTopMiddle.src = \''.HOME.'/sawanna/images/window_middle.gif\';';
                                       $html.='SawannaWindowBtn.src = "'.HOME.'/sawanna/images/window_right.gif";';
                                       $html.='SawannaWindowBtnOver.src = "'.HOME.'/sawanna/images/window_right_over.gif";';
                                       $html.='</script>';
			$html.='<script language="JavaScript" src="'.HOME.'/js/window.js"></script>';
			$this->javascript['window']=true;
		}
		$html.='<script language="JavaScript">';
		$html.='SawannaWindow.construct("'.$width.'","'.$height.'","'.addslashes($content).'","'.addslashes($title).'","'.$display.'","'.$hideonly.'");';
		$html.='</script>';
		
		return $html;
	}
	
	function construct_admin_panel()
	{
		global $sawanna_admin_panel_elements;
		
		$html='';
		if (!$this->javascript['window'])
		{
		             $html.='<script language="JavaScript">';
		             $html.='SawannaWindowTopLeft = new Image();';
                                       $html.='SawannaWindowTopMiddle = new Image();';
                                       $html.='SawannaWindowBtn = new Image();';
                                       $html.='SawannaWindowBtnOver = new Image();';
                                       $html.='SawannaWindowTopLeft.src = \''.HOME.'/sawanna/images/window_left.gif\';';
                                       $html.='SawannaWindowTopMiddle.src = \''.HOME.'/sawanna/images/window_middle.gif\';';
                                       $html.='SawannaWindowBtn.src = "'.HOME.'/sawanna/images/window_right.gif";';
                                       $html.='SawannaWindowBtnOver.src = "'.HOME.'/sawanna/images/window_right_over.gif";';
                                       $html.='</script>';
			$html.='<script language="JavaScript" src="'.HOME.'/js/window.js"></script>';
			$this->javascript['window']=true;
		}
		if (!$this->javascript['cajax'])
		{
			$html.='<script language="JavaScript" src="'.HOME.'/js/cajax.js"></script>';
			$this->javascript['cajax']=true;
		}

		$html.='<script language="JavaScript">';
		$html.='admin_panel_btn=new Image();';
		$html.='admin_panel_btn.src="'.HOME.'/sawanna/images/admin_panel_btn.gif";';
		$html.='function RequestContent(url,element) { var ajax=new CAJAX(); ajax.url=url; ajax.mode="json"; ajax.update=function() { ajax.updateReadyState(); if (ajax.response != null) { document.getElementById(element).innerHTML=ajax.response.html; if (ajax.response.exec.length > 0) eval(ajax.response.exec); }; }; ajax.send(); };';
		$html.='function Admin_Panel_Assign_Window(width,height,title,description,id) { if (!document.getElementById(id)) { SawannaWindow.construct(width,height,\'<div style="font-size:12px;font-weight:bold">\'+description+\'</div><div style="height:\'+(parseInt(height)-65)+\';border:1px solid #555555;background-color:#F8F8F6;padding:5px;margin-top:10px"><div id="\'+id+\'"><table width="100%" cellpadding="0" cellspacing="0"><tr><td width="100%" height="\'+(parseInt(height)-80)+\'" align="center"><img src="'.HOME.'/sawanna/images/waiter.gif" width="20" height="20" /></td></tr></table></div></div>\',title,1,0); } };';
		
		$html.='</script>';
			
		$content='<a href="'.$sawanna_admin_panel_elements['sawanna'].'" style="text-decoration:none"><div style="float:left;color:#EEEEEE;font-size:12px;font-family:Tahoma;margin-left:5px;cursor:pointer">Sawanna CMS</div></a>';
		
		foreach ($sawanna_admin_panel_elements as $name=>$element)
		{
			if (is_array($element))
			{
				if (is_array($element['workspace']))
				{
					if (in_array($this->workspace,$element['workspace']))
					{
						$content.='<div style="float:left;width:30px;height:30px;margin-top:5px;margin-bottom:5px;margin-left:5px;cursor:pointer" onmouseover="this.style.backgroundImage=\'url(\'+admin_panel_btn.src+\')\'" onmouseout="this.style.backgroundImage=\'\'"><img src="'.$element['btn'].'" width="30" height="30" onclick="AdminPanelPopupHide();Admin_Panel_Assign_Window(\''.$element['winWidth'].'\',\''.$element['winHeight'].'\',\''.$element['winTitle'].'\',\''.$element['desc'].'\',\'sawanna_admin_window_'.$element['name'].'\');RequestContent(\''.$element['url'].'\',\'sawanna_admin_window_'.$element['name'].'\');" onmouseover="AdminPanelPopupShow(event,\''.$element['desc'].'\')" onmouseout="AdminPanelPopupHide()" /></div>';
					}
				} elseif ($this->workspace == $element['workspace'])
				{
					$content.='<div style="float:left;width:30px;height:30px;margin-top:5px;margin-bottom:5px;margin-left:5px;cursor:pointer" onmouseover="this.style.backgroundImage=\'url(\'+admin_panel_btn.src+\')\'" onmouseout="this.style.backgroundImage=\'\'"><img src="'.$element['btn'].'" width="30" height="30" onclick="AdminPanelPopupHide();Admin_Panel_Assign_Window(\''.$element['winWidth'].'\',\''.$element['winHeight'].'\',\''.$element['winTitle'].'\',\''.$element['desc'].'\',\'sawanna_admin_window_'.$element['name'].'\');RequestContent(\''.$element['url'].'\',\'sawanna_admin_window_'.$element['name'].'\');" onmouseover="AdminPanelPopupShow(event,\''.$element['desc'].'\')" onmouseout="AdminPanelPopupHide()" /></div>';
				}
			} elseif ($name == 'logout')
			{
				$content.='<div style="float:right;color:#EEEEEE;font-size:12px;font-family:Tahoma;margin-right:5px"><a href="'.$element.'" style="color:EFEFEF;text-decoration:none"><img src="'.HOME.'/sawanna/images/exit.gif" width="30" height="30" align="absmiddle" border="0" style="margin-top:5px;margin-bottom:5px" />[string]logout[/string]</a></div>';
			}
		}
		
		$html.='<div id="sawanna_admin_panel" style="position:absolute;z-index:99;left:0px;top:0px;width:100%;height:80px;"  onclick="this.style.top=this.offsetTop+150"><div id="sawanna_admin_popup_panel" style="visibility:hidden;width:100%;height:40px;"><div id="sawanna_admin_popup" style="position:relative;top:0px;width:200px;height:40px;line-height:40px;background-image:url(\''.HOME.'/sawanna/images/admin_panel_popup.gif\');background-position:top left;background-repeat:no-repeat;color:#EEEEEE;font-size:12px;white-space:nowrap;text-align:center;overflow:hidden"></div></div><div style="width:100%;height:40px;line-height:40px;overflow:hidden;border-top:1px solid #EEEEEE;border-bottom:1px solid #EEEEEE;background-image:url(\''.HOME.'/sawanna/images/admin_panel.gif\');background-position:top left;background-repeat:repeat-x">'.$content.'</div></div>';
		$html.='<script language="JavaScript">';
		$html.='function AdminPanelPopupShow(event,popup) { var event=event || window.event; admin_popup_timer=window.setTimeout("document.getElementById(\'sawanna_admin_popup_panel\').style.visibility=\'visible\'",500); document.getElementById("sawanna_admin_popup").innerHTML=popup; document.getElementById("sawanna_admin_popup").style.left=event.clientX + document.body.scrollLeft - 100 + "px"; };';
		$html.='function AdminPanelPopupHide() {window.clearTimeout(admin_popup_timer); document.getElementById(\'sawanna_admin_popup_panel\').style.visibility=\'hidden\'; };';
		$html.='document.getElementById("sawanna_admin_panel").style.top=document.body.clientHeight+document.body.scrollTop;';
		$html.='window.setInterval("UpdateSawannaAdminPanelPos()",100);';
		$html.='function UpdateSawannaAdminPanelPos() { var sawanna_admin_panel_step=4; var sawanna_admin_panel_delta=document.body.clientHeight+document.body.scrollTop-document.getElementById("sawanna_admin_panel").offsetHeight-document.getElementById("sawanna_admin_panel").offsetTop; if (sawanna_admin_panel_delta > 0) sawanna_admin_panel_delta+=sawanna_admin_panel_step; else if (sawanna_admin_panel_delta < 0) sawanna_admin_panel_delta-=sawanna_admin_panel_step; document.getElementById("sawanna_admin_panel").style.top=document.getElementById("sawanna_admin_panel").offsetTop+Math.floor(sawanna_admin_panel_delta / sawanna_admin_panel_step ) +"px"; }';
		$html.='</script>';
		
		if (($body=strpos($this->html,'</body>')) !==false)
		{
			$this->html=substr($this->html,0,$body)."\n".$html."\n".substr($this->html,$body);
		}
	}
}
?>